import { sanitizeAlphaFloat } from '../helpers'

const rgbToHex = (redNum, greenNum, blueNum) => {
  let red = redNum.toString(16)
  let green = greenNum.toString(16)
  let blue = blueNum.toString(16)

  if (red.length === 1) { red = "0" + red }
  if (green.length === 1) { green = "0" + green }
  if (blue.length === 1) { blue = "0" + blue }
  return "" + red + green + blue
}

const rgbaToHexa = (redNum, greenNum, blueNum, alphaNum) => {
  let hex = rgbToHex(redNum, greenNum, blueNum)
  let alpha = Math.round(sanitizeAlphaFloat(alphaNum) * 255).toString(16)
  if (alpha.length == 1) { alpha = "0" + alpha }
  return hex + alpha
}

const rgbToHsl = (redNum, greenNum, blueNum) => {
  let hue = 0, saturation = 0, lightness = 0

  let redFrac = Number.parseFloat(redNum / 255),
    greenFrac = Number.parseFloat(greenNum / 255),
    blueFrac = Number.parseFloat(blueNum / 255)

  let channelMin = Math.min(redFrac, blueFrac, greenFrac),
    channelMax = Math.max(redFrac, blueFrac, greenFrac),
    delta = channelMax - channelMin

  if (delta === 0) { hue = 0 }
  // Red is the maximum color
  else if (channelMax === redFrac) {
    hue = ((greenFrac - blueFrac) / delta) % 6
  }
  // Green is the maximum color
  else if (channelMax === greenFrac) {
    hue = (blueFrac - redFrac) / delta + 2
  }
  // Green is the maximum color
  else {
    hue = (redFrac - greenFrac) / delta + 4
  }

  hue = Math.round(hue * 60)
  // Make negative hues positive behind 360°
  if (hue < 0) { hue += 360 }

  lightness = (channelMax + channelMin) / 2
  if (delta === 0) { saturation = 0 }
  else { saturation = delta / (1 - Math.abs(2 * lightness - 1)) }

  lightness = +(lightness * 100).toFixed(1)
  saturation = +(saturation * 100).toFixed(1)

  return { "hue": hue, "saturation": saturation, "lightness": lightness }
}

const rgbaToHsla = (redNum, greenNum, blueNum, alphaNum) => {
  let hsl = rgbToHsl(redNum, greenNum, blueNum)
  hsl.alpha = sanitizeAlphaFloat(alphaNum)
  return hsl
}

export { rgbToHex, rgbaToHexa, rgbToHsl, rgbaToHsla }
